# Generated by Django 3.0.2 on 2020-04-23 00:54

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('drf_schemas', '0002_auto_20200422_1811'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='itemfield',
            name='create_target',
        ),
    ]
