# Generated by Django 3.0.2 on 2020-04-22 23:11

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('drf_schemas', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='item',
            name='name',
        ),
        migrations.AddField(
            model_name='itemfield',
            name='create_target',
            field=models.BooleanField(blank=True, default=True),
        ),
    ]
